@@ -226,7 +226,7 @@ static void addfile(const char *name, const char *suffix)
 }
 
 /* listfile returns non-zero if the file is a directory */
-static int listfile(const PureFileInfo * const fi,  const char *name)
+static int listfile(const PureFileInfo * const fi, const char *name)
 {
     int rval = 0;
     struct stat st;
@@ -768,6 +768,32 @@ static void listdir(unsigned int depth, int f, void * const tls_fd,
     names = NULL;
 }
 
+static char *unescape_and_return_next_file(char * const str) {
+    char *pnt = str;
+    signed char seen_backslash = 0;    
+    
+    while (*pnt != 0) {
+        if (seen_backslash == 0) {
+            if (*pnt == '\\') {
+                seen_backslash = 1;
+            } else if (*pnt == ' ') {
+                *pnt++ = 0;
+                if (*pnt != 0) {
+                    return pnt;
+                }
+                break;
+            }
+            pnt++;            
+        } else {
+            seen_backslash = 0;
+            if (*pnt == ' ' || *pnt == '\\') {
+                memmove(pnt - 1, pnt, strlen(pnt) + (size_t) 1U);
+            }
+        }
+    }
+    return NULL;
+}
+
 void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
              const int opt_a_, const int split_args)
 {
@@ -870,8 +896,7 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
 
             if (split_args == 0) {
                 endarg = NULL;
-            } else if ((endarg = strchr(arg, ' ')) != NULL) {
-                *endarg++ = 0;
+            } else if ((endarg = unescape_and_return_next_file(arg)) != NULL) {
                 justone = 0;
             }
 #ifdef DEBUG
@@ -885,16 +910,20 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
             alarm(GLOB_TIMEOUT);
             a = sglob(arg,
                       GLOB_BRACE | (opt_a ? GLOB_PERIOD : 0),
-                     NULL, &g, max_ls_files + 2, max_ls_depth * 2);
+                      NULL, &g, max_ls_files + 2, max_ls_depth * 2);
             alarm(0);
             if (a == 0) {
                 char **path;
 
-                path = g.gl_pathv;
-                if (path && path[0] && path[1]) {
+                if (g.gl_pathc <= 0) {
+                    path = NULL;
+                } else {
+                    path = g.gl_pathv;
+                }
+                if (path != NULL && path[0] != NULL && path[1] != NULL) {
                     justone = 0;
                 }
-                while (path && *path) {
+                while (path != NULL && *path != NULL) {
                     struct stat st;
 
                     if (stat(*path, &st) == 0) {
@@ -909,11 +938,11 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
                 }
                 outputfiles(c, tls_fd);    /* in case of opt_C */
                 path = g.gl_pathv;
-                while (path && *path) {
+                while (path != NULL && *path != NULL) {
                     if (matches >= max_ls_files) {
                         break;
                     }
-                    if (**path) {
+                    if (**path != 0) {
                         if (!justone) {
 #ifdef WITH_RFC2640                            
                             c_buf = charset_fs2client(*path);
